﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectSeaplane.Entities;
using ProjectSeaplane.MovementStrategy;
using static ProjectSeaplane.Direction;
namespace ProjectSeaplane.DrawningObjects
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningPlane
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityPlane? EntityPlane { get; protected set; }
        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// Левая координата прорисовки автомобиля
        /// </summary>
        protected int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки автомобиля
        /// </summary>
        protected int _startPosY;
        /// <summary>
        /// Ширина прорисовки автомобиля
        /// </summary>
        private int _planeWidth = 200;
        /// <summary>
        /// Высота прорисовки автомобиля
        /// </summary>
        private int _planeHeight = 90;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public DrawningPlane(int speed, double weight, Color bodyColor, int width, int height)
        {
            // TODO: Продумать проверки

            if (width > _planeWidth && height > _planeHeight)
            {
                _pictureWidth = width;
                _pictureHeight = height;
                EntityPlane = new EntityPlane(speed, weight, bodyColor);
            }
        }
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <param name="planeWidth">Ширина прорисовки автомобиля</param>
        /// <param name="planeHeight">Высота прорисовки автомобиля</param>
        protected DrawningPlane(int speed, double weight, Color bodyColor, int width, int height, int planeWidth, int planeHeight)
        {
            // TODO: Продумать проверки

            if (width > _planeWidth && height > _planeHeight)
            {
                _pictureWidth = width;
                _pictureHeight = height;
                _planeWidth = planeWidth;
                _planeHeight = planeHeight;
                EntityPlane = new EntityPlane(speed, weight, bodyColor);
            }


        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if ((x > 0) && (x < _pictureWidth - _planeWidth))
                _startPosX = x;
            else _startPosX = 0;
            if ((y > 0) && (y < _pictureHeight - _planeHeight))
                _startPosY = y;

            else _startPosY = 0;
            _startPosX = x;
            _startPosY = y;
        }

        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _planeWidth;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _planeHeight;

        /// <summary>
        /// Проверка, что объект может переместится по указанному направлению
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - можно переместится по указанному направлению</returns>
        public bool CanMove(DirectionType direction)
        {
            if (EntityPlane == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityPlane.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityPlane.Step > 0,
                // вправо
                DirectionType.Right => _startPosX + _planeWidth + EntityPlane.Step < _pictureWidth,
                //вниз
                DirectionType.Down => _startPosY + _planeHeight + EntityPlane.Step < _pictureHeight,
            };

        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityPlane == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    _startPosX -= (int)EntityPlane.Step;
                    break;
                //вверх
                case DirectionType.Up:
                    _startPosY -= (int)EntityPlane.Step;
                    break;
                // вправо
                case DirectionType.Right:
                    _startPosX += (int)EntityPlane.Step;
                    break;
                //вниз
                case DirectionType.Down:
                    _startPosY += (int)EntityPlane.Step;
                    break;

            }
        }

        /// <summary>
        /// Получение объекта IMoveableObject из объекта DrawningCar
        /// </summary>
        public IMoveableObject GetMoveableObject => new DrawningObjectPlane(this);

        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityPlane == null)
            {
                return;
            }

            Brush BodyBrush = new SolidBrush(EntityPlane.BodyColor);
            Pen pen = new(Color.Black);

            g.DrawLine(pen, _startPosX + 10, _startPosY, _startPosX + 10, _startPosY + 30);
            g.DrawLine(pen, _startPosX + 10, _startPosY, _startPosX + 40, _startPosY + 30);
            g.DrawLine(pen, _startPosX + 10, _startPosY + 30, _startPosX + 150, _startPosY + 30);
            g.DrawLine(pen, _startPosX + 150, _startPosY + 30, _startPosX + 150, _startPosY + 80);
            g.DrawLine(pen, _startPosX + 10, _startPosY + 80, _startPosX + 150, _startPosY + 80);
            g.DrawLine(pen, _startPosX + 150, _startPosY + 30, _startPosX + 200, _startPosY + 55);
            g.DrawLine(pen, _startPosX + 200, _startPosY + 55, _startPosX + 150, _startPosY + 55);
            g.DrawLine(pen, _startPosX + 200, _startPosY + 55, _startPosX + 150, _startPosY + 80);
            g.DrawPie(pen, _startPosX, _startPosY + 30, 20, 50, 90, 180);
            g.DrawEllipse(pen, _startPosX, _startPosY + 25, 35, 10);
            g.DrawEllipse(pen, _startPosX + 40, _startPosY + 50, 80, 10);

            //вертикальная стабилизация
            Brush mainBrush = new SolidBrush(EntityPlane.BodyColor);
            Point point7 = new Point(_startPosX + 11, _startPosY + 1);
            Point point8 = new Point(_startPosX + 11, _startPosY + 31);
            Point point9 = new Point(_startPosX + 39, _startPosY + 31);
            Point[] vert_stab = { point7, point8, point9, point7 };
            g.FillPolygon(mainBrush, vert_stab);

            //хвостовая часть
            g.FillPie(mainBrush, _startPosX, _startPosY + 30, 20, 50, 90, 180);
            //нос
            Brush brGray = new SolidBrush(Color.Gray);
            Point point1 = new Point(_startPosX + 150, _startPosY + 30);
            Point point2 = new Point(_startPosX + 200, _startPosY + 55);
            Point point3 = new Point(_startPosX + 150, _startPosY + 55);
            Point[] nos1 = { point1, point2, point3, point1 };
            g.FillPolygon(brGray, nos1);
            Point point4 = new Point(_startPosX + 150, _startPosY + 55);
            Point point5 = new Point(_startPosX + 200, _startPosY + 55);
            Point point6 = new Point(_startPosX + 150, _startPosY + 80);
            Point[] nos2 = { point4, point5, point6, point4 };
            g.FillPolygon(brGray, nos2);
            g.DrawLine(pen, _startPosX + 200, _startPosY + 55, _startPosX + 150, _startPosY + 55);

            //корпус
            Point point10 = new Point(_startPosX + 10, _startPosY + 31);
            Point point11 = new Point(_startPosX + 149, _startPosY + 31);
            Point point12 = new Point(_startPosX + 149, _startPosY + 79);
            Point point13 = new Point(_startPosX + 10, _startPosY + 79);
            Point[] body = { point10, point11, point12, point13, point10 };
            g.FillPolygon(mainBrush, body);

            //крылья
            Brush brBlack = new SolidBrush(Color.Black);
            g.FillEllipse(brBlack, _startPosX + 40, _startPosY + 50, 80, 10);
            g.FillEllipse(brBlack, _startPosX, _startPosY + 25, 35, 10);
        }
    }
}
