using ProjectSeaplane.DrawningObjects;
using ProjectSeaplane.MovementStrategy;
using static ProjectSeaplane.Direction;
using System;

namespace ProjectSeaplane
{
    public partial class FormSeaplane : Form
    {

        private DrawningPlane? _drawningSeaplane;
        /// <summary>
        ///  
        /// </summary>
        private AbstractStrategy? _abstractStrategy;
        /// <summary>
        ///  
        /// </summary>
        public DrawningPlane? SelectedPlane { get; private set; }
        public FormSeaplane()
        {
            InitializeComponent();
            ComboBox.Items.Add(0);
            ComboBox.Items.Add(1);
            _abstractStrategy = null;
            SelectedPlane = null;
        }

        private void Draw()
        {
            if (_drawningSeaplane == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxSeaplane.Width, pictureBoxSeaplane.Height);
            Graphics g = Graphics.FromImage(bmp);
            _drawningSeaplane.DrawTransport(g);
            pictureBoxSeaplane.Image = bmp;
        }

        private void buttonCreatePlane_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _drawningSeaplane = new DrawningPlane(random.Next(100, 300), random.Next(1000, 3000),
                color, pictureBoxSeaplane.Width, pictureBoxSeaplane.Height);
            _drawningSeaplane.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        private void buttonMove_Click(object sender, EventArgs e)
        {
            if (_drawningSeaplane == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningSeaplane.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawningSeaplane.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawningSeaplane.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawningSeaplane.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();

        }

        private void buttonCreateSeaplane_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));

            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }

            Color dopColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));

            ColorDialog dialog2 = new();
            if (dialog2.ShowDialog() == DialogResult.OK)
            {
                dopColor = dialog2.Color;
            }
            _drawningSeaplane = new DrawningSeaplane(random.Next(100, 300), random.Next(1000, 3000),
            color, dopColor, true, true,
                pictureBoxSeaplane.Width, pictureBoxSeaplane.Height);
            _drawningSeaplane.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawningSeaplane == null)
            {
                return;
            }
            if (ComboBox.Enabled)
            {
                _abstractStrategy = ComboBox.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),

                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new DrawningObjectPlane(_drawningSeaplane), pictureBoxSeaplane.Width, pictureBoxSeaplane.Height);
                ComboBox.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                ComboBox.Enabled = true;
                _abstractStrategy = null;
            }
        }

        private void buttonSelectPlane_Click(object sender, EventArgs e)
        {
            SelectedPlane = _drawningSeaplane;
            DialogResult = DialogResult.OK;
        }
    }
}